//
//  Gateway.swift
//  UIDirectoryOnline
//
//  Created by Joe Gregg on 10/27/22.
//

import Foundation

class User : Codable {
    var name : String
    var password : String
    
    init(name n : String,password p : String) {
        name = n
        password = p
    }
}

class Gateway : ObservableObject {
    @Published var people : [Person] = []
    var key : String = ""
    
    func newUser(name n: String,password p: String) async -> Bool {
        let u : User = User(name:n,password:p)
        let urlStr = "https://cmsc106.net/directory/users"
        guard let url = URL(string: urlStr) else { return false }
        var request = URLRequest(url: url)
        request.httpMethod = "POST"
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        do {
            let jsonData = try JSONEncoder().encode(u)
            let (data, response) = try await URLSession.shared.upload(for: request, from: jsonData)
                
            guard (response as! HTTPURLResponse).statusCode == 200 else { return false }
            key = try JSONDecoder().decode(String.self, from: data)
            return true
        } catch {
        }
        return false
    }

    func login(name n: String,password p: String) async -> Bool {
        let urlStr = "https://cmsc106.net/directory/users?user=\(n)&password=\(p)"
        guard let url = URL(string: urlStr) else { return false}
        do {
            let (data, response) = try await URLSession.shared.data(from: url)
            guard (response as! HTTPURLResponse).statusCode == 200 else { return false }
            key = try JSONDecoder().decode(String.self, from: data)
            await refresh()
            return true
        } catch {
        }
        return false
    }
    
    func refresh() async {
        guard !key.isEmpty else { return }
        let urlStr = "https://cmsc106.net/directory/people/" + key
        guard let url = URL(string: urlStr) else { return }
        do {
            let (data, response) = try await URLSession.shared.data(from: url)
            guard (response as! HTTPURLResponse).statusCode == 200 else { return }
            let pl = try JSONDecoder().decode([Person].self, from: data)
            DispatchQueue.main.async { self.people = pl }
        } catch {
        }
    }
    
    func addPerson(_ p : Person) async {
        guard !key.isEmpty else { return }
        let urlStr = "https://cmsc106.net/directory/people/\(key)"
        guard let url = URL(string: urlStr) else { return }
        var request = URLRequest(url: url)
        request.httpMethod = "POST"
        request.setValue("application/json", forHTTPHeaderField: "Content-Type")
        do {
            let jsonData = try JSONEncoder().encode(p)
            let (_, response) = try await URLSession.shared.upload(for: request, from: jsonData)
                
            guard (response as! HTTPURLResponse).statusCode == 200 else { return  }
            await refresh()
        } catch {
        }
    }

    func removePerson(at n : Int) async {
        guard !key.isEmpty else { return }
        let toRemove = people[n]
        let urlStr = "https://cmsc106.net/directory/people/\(key)/\(toRemove.id)"
        guard let url = URL(string: urlStr) else { return }
        var request = URLRequest(url: url)
        request.httpMethod = "DELETE"
        do {
            let (_, response) = try await URLSession.shared.data(for: request)
            guard (response as! HTTPURLResponse).statusCode == 200 else { return }
            await refresh()
        } catch {
        }
    }
}
